# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from freezer_dr.fencers.common.driver import FencerBaseDriver
from freezer_dr.fencers.drivers.ipmi.ipmitool import IpmiInterface
from oslo_log import log
from oslo_config import cfg

CONF = cfg.CONF
LOG = log.getLogger(__name__)


class IpmiDriver(FencerBaseDriver):
    def __init__(self, node, **kwargs):
        super(IpmiDriver, self).__init__(node, **kwargs)
        self.interface = IpmiInterface(
            node.get('fencer-ip'),
            node.get('fencer-user'),
            node.get('fencer-password'),
            verbose=CONF.debug)

    def force_shutdown(self):
        try:
            self.interface.power_down()
        except Exception as e:
            LOG.error(e)

    def graceful_shutdown(self):
        try:
            self.interface.power_soft()
        except Exception as e:
            LOG.error(e)

    def status(self):
        return self.interface.get_power_status()

    # @todo remove this fn as it's for testing purposes only :)
    def power_on(self):
        self.interface.power_on()

    def get_info(self):
        return {
            'name': 'IPMI Interface driver',
            'version': 1.0,
            'author': 'Hewlett-Packard Development Company, L.P'
        }


