# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from freezer_dr.fencers.common.driver import FencerBaseDriver
from oslo_log import log
from oslo_config import cfg
import libvirt

CONF = cfg.CONF
LOG = log.getLogger(__name__)


class LibvirtDriver(FencerBaseDriver):
    def __init__(self, node, **kwargs):
        super(LibvirtDriver, self).__init__(node, **kwargs)
        conn_name = kwargs.get('name', None)
        self.connection = libvirt.open(name=conn_name)

    def force_shutdown(self):
        target = self.connection.lookupByName(name=self.node.get('domain-name'))
        return target.destroy()

    def graceful_shutdown(self):
        target = self.connection.lookupByName(name=self.node.get('domain-name'))
        return target.shutdown()

    def status(self):
        target = self.connection.lookupByName(name=self.node.get('domain-name'))
        return target.isActive()

    def get_info(self):
        return {
            'name': 'Libvirt Interface driver',
            'version': 1.0,
            'author': 'Hewlett-Packard Development Company, L.P'
        }


